"""
@file Reines.py
@brief Le programme résoud un échiquier.
@details Le programme applique l'algorithme de résolution BFS avec deux types de grilles : 6x6 et 8x8.

@author TORRES Lucas
@version 0.4
@date 2024-03-14
"""

import time
from queue import Queue

# Vérifie si le placement d'une reine est valide par rapport aux reines déjà placées
def est_valide(placement, nouvelle_colonne):
    nouvelle_ligne = len(placement)
    for ligne in range(nouvelle_ligne):
        # Vérifie les règles
        if placement[ligne] == nouvelle_colonne or \
           placement[ligne] - ligne == nouvelle_colonne - nouvelle_ligne or \
           placement[ligne] + ligne == nouvelle_colonne + nouvelle_ligne:
            return False
    return True

# Utilise l'algorithme BFS 
def bfs_n_reines(n):
    solutions = []
    file = Queue()
    file.put([])  # Commence avec un plateau vide
    
    while not file.empty():
        placement = file.get()
        
        if len(placement) == n:
            solutions.append(placement) 
            continue
        
        for colonne in range(n):
            # Teste la validité de l'ajout d'une reine dans chaque colonne
            if est_valide(placement, colonne):
                file.put(placement + [colonne])
                
    return solutions

# Affiche les solutions trouvées
def afficher_solutions(solutions):
    for index, solution in enumerate(solutions, start=1):
        print(f"Solution {index}:")
        for i in solution:
            ligne = ['.'] * len(solution)
            ligne[i] = 'Q'  
            print(' '.join(ligne))
        print("\n")

# Fonction principale 
def main():
    tailles = [6, 8]  # Tailles d'échiquier 
    for n in tailles:
        temps_debut = time.time()
        solutions = bfs_n_reines(n)  
        temps_fin = time.time()

        print(f"Algorithme 2 - BFS :")
        print(f"Pour un échiquier (GRILLE {n}x{n}) :")
        print(f"\033[92mNombre de solutions trouvées : {len(solutions)}\033[0m")
        print(f"\033[93mTemps de résolution pour les solutions: {temps_fin - temps_debut:.4f} secondes.\033[0m\n")
        print("\033[94m----------------------------------------------------------\n\033[0m")
        
    for n in tailles:
        afficher = input(f"\033[96mVoulez-vous afficher les solutions pour l'échiquier {n}x{n} ? (oui/non): \033[0m").strip().lower()
        if afficher == "oui":
            print(f"\033[97mSolutions pour l'échiquier {n}x{n} :\033[0m")
            solutions = bfs_n_reines(n)  
            afficher_solutions(solutions)

if __name__ == "__main__":
    main()
