import time
# Fonction pour demander à l'utilisateur de saisir la taille du plateau de jeu
def demander_taille_plateau():
    return int(input("Entrez la taille du plateau de jeu (un nombre): "))


# Fonction pour vérifier si deux reines se menacent ou non
def est_en_securite(plateau, ligne, colonne):
    # Vérifie si deux reines se trouvent sur la même colonne et ligne
    for i in range(ligne):
        if plateau[i][colonne] == 'Q':
            return False

    # Vérifie si deux reines se trouvent sur la même diagonale vers le haut à gauche
    (i, j) = (ligne, colonne)
    while i >= 0 and j >= 0:
        if plateau[i][j] == 'Q':
            return False
        i -= 1
        j -= 1

    # Vérifie si deux reines se trouvent sur la même diagonale vers le haut à droite
    (i, j) = (ligne, colonne)
    while i >= 0 and j < len(plateau):
        if plateau[i][j] == 'Q':
            return False
        i -= 1
        j += 1

    return True


# Fonction pour afficher une solution du jeu de N reines
def afficher_solution(plateau):
    for ligne in plateau:
        print(str(ligne).replace(',', '').replace('\'', ''))
    print()


# Fonction récursive pour placer les reines sur le plateau
def placer_reines(plateau, ligne, solutions, compte_solutions):
    # Si toutes les reines sont placées avec succès, ajoute la solution et arrête
    if ligne == len(plateau):
        compte_solutions[0] += 1
        solutions.append([row[:] for row in plateau])  # Ajoute une copie de la solution trouvée
        return

    # Place une reine sur chaque case de la ligne actuelle
    # et explore toutes les possibilités
    for colonne in range(len(plateau)):
        # Si deux reines ne se menacent pas
        if est_en_securite(plateau, ligne, colonne):
            # Place une reine sur la case actuelle
            plateau[ligne][colonne] = 'Q'

            # Explore pour la ligne suivante
            placer_reines(plateau, ligne + 1, solutions, compte_solutions)

            # Retire la reine de la case actuelle et explore d'autres possibilités
            plateau[ligne][colonne] = '-'


# Fonction pour demander les coordonnées d'une reine à vérifier
def demander_coordonnees_plateau(taille):
    ligne = int(input(f"Entrez le numéro de ligne de la reine (1-{taille}): ")) - 1
    colonne = int(input(f"Entrez le numéro de colonne de la reine (1-{taille}): ")) - 1
    return ligne, colonne


# Fonction pour compter le nombre d'occurrences d'une reine dans les solutions
def compter_occurrences_reine(solutions, ligne, colonne):
    occurrences = 0
    for solution in solutions:
        if solution[ligne][colonne] == 'Q':
            occurrences += 1
    return occurrences


# Fonction pour afficher les résultats d'occurrences d'une reine
def afficher_resultats_occurrences(solutions, ligne, colonne):
    print(f"Résultats pour la reine située à la ligne {ligne + 1} et colonne {colonne + 1}:")
    for idx, solution in enumerate(solutions, 1):
        if solution[ligne][colonne] == 'Q':
            print("Solution", idx)
            afficher_solution(solution)


# Fonction principale
def jouer_jeu_reines():
    # Demande à l'utilisateur la taille du plateau de jeu
    taille = demander_taille_plateau()
    plateau = [['-' for _ in range(taille)] for _ in range(taille)]
    solutions = []
    compte_solutions = [0]

    # Résout le jeu des N reines
    debut = time.process_time()  # Temps de début
    placer_reines(plateau, 0, solutions, compte_solutions)
    fin = time.process_time()  # Temps de fin

    # Demande les coordonnées de la reine à vérifier
    ligne, colonne = demander_coordonnees_plateau(taille)
    occurrences = compter_occurrences_reine(solutions, ligne, colonne)
    print(f"La reine à la ligne {ligne + 1} et colonne {colonne + 1} apparaît {occurrences} fois dans {compte_solutions[0]} solutions.")

    # Demande si l'utilisateur veut afficher uniquement les solutions où cette reine apparaît
    afficher_seulement = input("Voulez-vous afficher uniquement les solutions où cette reine apparaît? (oui/non): ")
    if afficher_seulement.lower() == "oui":
        afficher_resultats_occurrences(solutions, ligne, colonne)
    
    # Demande si l'utilisateur veut afficher toutes les solutions
    afficher_toutes = input("Voulez-vous afficher toutes les solutions? (oui/non): ")
    if afficher_toutes.lower() == "oui":
        print("Toutes les solutions :")
        for idx, solution in enumerate(solutions, 1):
            print("Solution", idx)
            afficher_solution(solution)

    # Affiche le temps d'exécution
    temps_execution = fin - debut
    print(f"Temps d'exécution CPU: {temps_execution} secondes")


# Appel de la fonction principale pour jouer le jeu de N reines
jouer_jeu_reines()
