import matplotlib.pyplot as plt

class Echiquier:
    def __init__(self, taille, x=None, y=None):
        self.taille = taille
        self.echiquier = [[0] * taille for i in range(taille)] # tableau de taille (taille x taille) écrit en compréhension
        self.solutions = [] 
    
    def placer_reine(self, ligne, colonne):
        self.echiquier[ligne][colonne] = 1

    def retirer_reine(self, ligne, colonne):
        self.echiquier[ligne][colonne] = 0

    def est_menacee(self, ligne, colonne):
        for i in range(self.taille):
            if self.echiquier[ligne][i] == 1 or self.echiquier[i][colonne] == 1: # si une des cases de la ligne ou de la colonne contient une reine
                return True

        for i in range(self.taille):
            for j in range(self.taille):
                if (i + j == ligne + colonne or i - j == ligne - colonne) and self.echiquier[i][j] == 1: # si l'addition ou la soustraction des coordonnées de deux cases est égale à la somme ou la différence des coordonnées de la reine 
                    return True

        return False
    
    
    def resoudre(self, colonne=0):
        
        """
        Fonction récursive pour placer les reines sur l'échiquier
        """

        if colonne >= self.taille: # condition d'arrêt, si on a placé toutes les reines (puisqu'on place une reine par colonne maximum)
            self.solutions.append(self.copier_echiquier()) # on ajoute une copie de l'échiquier à la liste des solutions pour les afficher plus tard
            return
        for i in range(self.taille):
            if not self.est_menacee(i, colonne):
                self.placer_reine(i, colonne)
                self.resoudre(colonne + 1) # on appelle la fonction récursivement pour placer la reine suivante
                self.retirer_reine(i, colonne) # si la condition d'arrêt n'est pas remplie après avoir essayé de placer toutes les reines, on retire la reine pour essayer une autre position

    def copier_echiquier(self):
        return [row[:] for row in self.echiquier] # row[:] nous permet de copier les valeurs de la liste

    def afficher_solutions(self):
        for solution in self.solutions:
            for ligne in solution:
                print(' '.join('R' if case == 1 else '.' for case in ligne)) # on affiche R pour une reine et . pour une case vide (tout autre affichage marcherait aussi) en compréhension
            print("") # on laisse un espace entre chaque solution


    def afficher_solutions_graphique(self):
        # on crée une figure pour chaque solution
        for i, solution in enumerate(self.solutions): # on recupère les solutions et leur indice
            if i > 0:
                afficher_solution = input(f"Voulez-vous afficher la solution suivante ? (O/N) : ")
                if afficher_solution == "N" or afficher_solution == "n":
                    break
                else:
                    plt.matshow(solution) # on affiche la solution sous forme de matrice
                    plt.title(f"Solution {i+1}")
                    plt.show()
            else:
                plt.matshow(solution)
                plt.title(f"Solution {i+1}")
                plt.show()
    
    def afficher_solutions_graphique_donnees(self, solutions):
         # on crée une figure pour chaque solution
        for i, solution in enumerate(solutions): # on recupère les solutions et leur indice
            if i > 0:
                afficher_solution = input(f"Voulez-vous afficher la solution suivante ? (O/N) : ")
                if afficher_solution == "N" or afficher_solution == "n":
                    break
                else:
                    plt.matshow(solution) # on affiche la solution sous forme de matrice
                    plt.title(f"Solution {i+1}")
                    plt.show()
            else:
                plt.matshow(solution)
                plt.title(f"Solution {i+1}")
                plt.show()